@@ -1,4 +1,4 @@
-/* $Id: diction.c,v 1.7 2006/01/18 16:48:53 jonz Exp $ */
+/* $Id: diction.c,v 1.9 2006/01/22 03:11:22 jonz Exp $ */
 
 /*
  DSPAM
@@ -131,7 +131,6 @@ ds_diction_term_create (ds_key_t key, const char *name)
     if (name)
       term->name = strdup(name);
   }
-
   return term;
 }
 
@@ -152,7 +151,11 @@ ds_diction_find (ds_diction_t diction, ds_key_t key)
 }
 
 ds_term_t
-ds_diction_touch (ds_diction_t diction, ds_key_t key, const char *name, int flags)
+ds_diction_touch(
+  ds_diction_t diction, 
+  ds_key_t key, 
+  const char *name, 
+  int flags)
 {
   unsigned long bucket = key % diction->size;
   ds_term_t parent = NULL;
@@ -160,7 +163,6 @@ ds_diction_touch (ds_diction_t diction, ds_key_t key, const char *name, int flag
   ds_term_t term;
 
   term = diction->tbl[bucket];
-
   while (term) {
     if (key == term->key) {
       insert = term;
@@ -254,7 +256,6 @@ ds_diction_next (ds_cursor_t cur)
     return NULL;
 
   term = cur->iter_next;
-
   if (term) {
     cur->iter_next = term->next;
     return term;
