@@ -1,4 +1,4 @@
-/* $Id: libdspam.c,v 1.148 2006/01/18 16:48:53 jonz Exp $ */
+/* $Id: libdspam.c,v 1.152 2006/01/28 17:23:50 jonz Exp $ */
 
 /*
  DSPAM
@@ -649,7 +649,7 @@ dspam_getsource (
   char *buf,
   size_t size)
 {
-  ds_message_block_t current_block;
+  ds_message_part_t current_block;
   ds_header_t current_heading = NULL;
   struct nt_node *node_nt;
   struct nt_c c;
@@ -662,7 +662,7 @@ dspam_getsource (
   if (node_nt == NULL)
     return EINVAL;
 
-  current_block = (ds_message_block_t) node_nt->ptr;
+  current_block = (ds_message_part_t) node_nt->ptr;
 
   node_nt = c_nt_first (current_block->headers, &c);
   while (node_nt != NULL)
@@ -769,7 +769,7 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
 
   /* Create our diction (lexical data in message) and patterns */
 
-  ds_diction_t diction = ds_diction_create(24593);
+  ds_diction_t diction = ds_diction_create(24593ul);
   ds_diction_t bnr_patterns = NULL;
   ds_term_t ds_term;
   ds_cursor_t ds_c;
@@ -1080,162 +1080,7 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
       CTX->totals.innocent_classified++;
   }
 
-  /* Increment and Store Tokens */
-
-  i = 0;
-  ds_c = ds_diction_cursor(diction);
-  ds_term = ds_diction_next(ds_c);
-  while(ds_term) {
-    unsigned long long crc;
-
-    crc = ds_term->key;
-
-    /* Create a signature if we're processing a message */
-
-    if ((!(CTX->flags & DSF_SBPH))  && 
-        CTX->flags & DSF_SIGNATURE && 
-       (CTX->operating_mode != DSM_CLASSIFY || !(CTX->_sig_provided)))
-    {
-      struct _ds_signature_token t;
-
-      memset(&t, 0, sizeof(t));
-      t.token = crc;
-      t.frequency = ds_term->frequency;
-      memcpy ((char *) CTX->signature->data +
-              (i * sizeof (struct _ds_signature_token)), &t,
-              sizeof (struct _ds_signature_token));
-    }
-
-    /* If classification was provided, force probabilities */
-    if (CTX->classification == DSR_ISSPAM)
-      ds_term->s.probability = 1.00;
-    else if (CTX->classification == DSR_ISINNOCENT) 
-      ds_term->s.probability = 0.00;
-
-    if (ds_term->type == 'D' &&
-        ( CTX->training_mode != DST_TUM  || 
-          CTX->source == DSS_ERROR       ||
-          CTX->source == DSS_INOCULATION ||
-          ds_term->s.spam_hits + ds_term->s.innocent_hits < 50 ||
-          ds_term->key == diction->whitelist_token             ||
-          CTX->confidence < 0.70))
-    {
-        ds_term->s.status |= TST_DIRTY;
-    }
-
-    if (ds_term->type == 'B' &&
-        CTX->totals.innocent_learned + CTX->totals.innocent_classified > 500 &&
-        CTX->flags & DSF_NOISE &&
-        CTX->_sig_provided == 0)
-    {
-        ds_term->s.status |= TST_DIRTY;
-    }
-
-    /* SPAM */
-    if (CTX->result == DSR_ISSPAM)
-    {
-      /* Inoculations increase token count considerably */
-      if (CTX->source == DSS_INOCULATION)
-      {
-        if (ds_term->s.innocent_hits < 2 && ds_term->s.spam_hits < 5)
-          ds_term->s.spam_hits += 5;
-        else
-          ds_term->s.spam_hits += 2;
-      }
-
-      /* Standard increase */
-      else
-      {
-        if (CTX->flags & DSF_UNLEARN) {
-          if (CTX->classification == DSR_ISSPAM)
-          {
-            if (_ds_match_attribute(CTX->config->attributes, 
-              "ProcessorWordFrequency", "occurrence"))
-            {
-              ds_term->s.spam_hits -= ds_term->frequency;
-              if (ds_term->s.spam_hits < 0)
-                ds_term->s.spam_hits = 0;
-            } else {
-              ds_term->s.spam_hits -= (ds_term->s.spam_hits>0) ? 1:0;
-            }
-          }
-        } else {
-          if (_ds_match_attribute(CTX->config->attributes,
-             "ProcessorWordFrequency", "occurrence"))
-          {
-            ds_term->s.spam_hits += ds_term->frequency;
-          } else {
-            ds_term->s.spam_hits++;
-          }
-        }
-      }
-
-      if (SPAM_MISS(CTX) && 
-          !(CTX->flags & DSF_UNLEARN) && 
-          CTX->training_mode != DST_TOE &&
-          CTX->training_mode != DST_NOTRAIN) 
-      { 
-        if (_ds_match_attribute(CTX->config->attributes,
-           "ProcessorWordFrequency", "occurrence"))
-        {
-          ds_term->s.innocent_hits -= ds_term->frequency;
-          if (ds_term->s.innocent_hits < 0)
-            ds_term->s.innocent_hits = 0;
-        } else {
-          ds_term->s.innocent_hits -= (ds_term->s.innocent_hits>0) ? 1:0;
-        }
-      }
-    }
-
-    /* INNOCENT */
-    else
-    {
-      if (CTX->flags & DSF_UNLEARN) { 
-        if (CTX->classification == DSR_ISINNOCENT)
-        {
-          if (_ds_match_attribute(CTX->config->attributes,
-             "ProcessorWordFrequency", "occurrence"))
-          {
-            ds_term->s.innocent_hits -= ds_term->frequency;
-            if (ds_term->s.innocent_hits < 0)
-              ds_term->s.innocent_hits = 0;
-          } else {
-            ds_term->s.innocent_hits -= (ds_term->s.innocent_hits>0) ? 1:0;
-          }
-        }
-      } else {
-        if (_ds_match_attribute(CTX->config->attributes,
-         "ProcessorWordFrequency", "occurrence"))
-        {
-          ds_term->s.innocent_hits += ds_term->frequency;
-        } else {
-          ds_term->s.innocent_hits++;
-        }
-      }
-
-      if (FALSE_POSITIVE(CTX)         && 
-          !(CTX->flags & DSF_UNLEARN) && 
-          CTX->training_mode != DST_TOE &&
-          CTX->training_mode != DST_NOTRAIN)
-      {
-
-        if (_ds_match_attribute(CTX->config->attributes,
-          "ProcessorWordFrequency", "occurrence"))
-        {
-          ds_term->s.spam_hits -= ds_term->frequency;
-          if (ds_term->s.spam_hits < 0)
-            ds_term->s.spam_hits = 0;
-        } else {
-          ds_term->s.spam_hits -= (ds_term->s.spam_hits>0) ? 1:0;
-        }
-
-      }
-    }
-
-    ds_term = ds_diction_next(ds_c);
-    i++;
-  }
-  ds_diction_close(ds_c);
+  _ds_increment_tokens(CTX, diction);
 
   /* Store all tokens */
   if (CTX->training_mode != DST_NOTRAIN) {
@@ -1294,7 +1139,7 @@ _ds_process_signature (DSPAM_CTX * CTX)
 {
   struct _ds_signature_token t;
   int num_tokens, i;
-  ds_diction_t diction = ds_diction_create(24593);
+  ds_diction_t diction = ds_diction_create(24593ul);
   ds_term_t ds_term;
   ds_cursor_t ds_c;
 
@@ -1582,11 +1427,8 @@ _ds_calc_stat (
 
   if (CTX->classification == DSR_ISSPAM)
     s->probability = .7;
-  else {
+  else 
     s->probability = (CTX->algorithms & DSP_MARKOV) ? .5 : .4;
-  }
-
-  
 
   /* Markovian Weighting */
 
@@ -1621,11 +1463,11 @@ _ds_calc_stat (
   /* Graham and Robinson Start Here */
 
   } else {
-    if (CTX->totals.spam_learned > 0 && 
-        CTX->totals.innocent_learned > 0)
-// &&
-//          ((s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) +
-//           (s->innocent_hits * 1.0 / CTX->totals.innocent_learned * 1.0)) > 0)
+    int ih = 1;
+    if (CTX->flags & DSF_BIAS)
+      ih = 2;
+
+    if (CTX->totals.spam_learned > 0 && CTX->totals.innocent_learned > 0)
     {
       if (token_type == DTT_BNR) {
         s->probability =
@@ -1633,28 +1475,44 @@ _ds_calc_stat (
           ((s->spam_hits * 1.0 / bnr_tot->spam_hits * 1.0) +
            (s->innocent_hits * 1.0 / bnr_tot->innocent_hits * 1.0));
       } else {
-
-        if (CTX->flags & DSF_BIAS)
-          s->probability =
-            (s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) /
-            ((s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) +
-             (s->innocent_hits * 2.0 / CTX->totals.innocent_learned * 1.0));
-        else
-          s->probability =
-            (s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) /
-            ((s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) +
-             (s->innocent_hits * 1.0 / CTX->totals.innocent_learned * 1.0));
+        s->probability =
+          (s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) /
+          ((s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) +
+           (s->innocent_hits * ih * 1.0 / CTX->totals.innocent_learned * 1.0));
       }
     }
 
-    if (s->spam_hits == 0 && s->innocent_hits > 0)
-    {
-        s->probability = 0.01;
+    if (s->spam_hits == 0 && s->innocent_hits > 0) {
+      s->probability = 0.01;
+      if (CTX->totals.spam_learned > 0 && CTX->totals.innocent_learned > 0)
+      {
+        if ((1.0 / CTX->totals.spam_learned * 1.0) /
+           ((1.0 / CTX->totals.spam_learned * 1.0) +
+           (s->innocent_hits * ih * 1.0 / CTX->totals.innocent_learned * 1.0))
+          < 0.01) 
+        {
+          s->probability = (1.0 / CTX->totals.spam_learned * 1.0) /
+           ((1.0 / CTX->totals.spam_learned * 1.0) +
+            (s->innocent_hits * ih *1.0 / CTX->totals.innocent_learned * 1.0));
+        }
+      }
     }
-    else if (s->spam_hits > 0 && s->innocent_hits == 0)
-    {
-        s->probability = 0.99;
+    else if (s->spam_hits > 0 && s->innocent_hits == 0) {
+      s->probability = 0.99;
+      if (CTX->totals.spam_learned > 0 && CTX->totals.innocent_learned > 0)
+      {
+        if ((s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) /
+           ((s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) +
+           (ih * 1.0 / CTX->totals.innocent_learned * 1.0))
+          > 0.99)
+        {
+          s->probability = (s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0)
+           / ((s->spam_hits * 1.0 / CTX->totals.spam_learned * 1.0) 
+           + (ih * 1.0 / CTX->totals.innocent_learned * 1.0));
+        }
+      }
     }
+
     if (  (CTX->flags & DSF_BIAS && 
           (s->spam_hits + (2 * s->innocent_hits) < min_hits))
        || (!(CTX->flags & DSF_BIAS) && 
@@ -1664,11 +1522,11 @@ _ds_calc_stat (
     }
   }
 
-  if (s->probability < 0.0100)
-    s->probability = 0.01;
+  if (s->probability < 0.0001)
+    s->probability = 0.0001;
 
-  if (s->probability > 0.9900)
-    s->probability = 0.99;
+  if (s->probability > 0.9999)
+    s->probability = 0.9999;
 
   /* Finish off Robinson */
 
@@ -2626,3 +2484,164 @@ ds_diction_t _ds_apply_bnr (DSPAM_CTX *CTX, ds_diction_t diction) {
 
   return bnr_patterns;
 }
+
+int _ds_increment_tokens(DSPAM_CTX *CTX, ds_diction_t diction) {
+  ds_cursor_t ds_c;
+  ds_term_t ds_term;
+  int i = 0;
+
+  ds_c = ds_diction_cursor(diction);
+  ds_term = ds_diction_next(ds_c);
+  while(ds_term) {
+    unsigned long long crc;
+
+    crc = ds_term->key;
+
+    /* Create a signature if we're processing a message */
+
+    if ((!(CTX->flags & DSF_SBPH))  && 
+        CTX->flags & DSF_SIGNATURE && 
+       (CTX->operating_mode != DSM_CLASSIFY || !(CTX->_sig_provided)))
+    {
+      struct _ds_signature_token t;
+
+      memset(&t, 0, sizeof(t));
+      t.token = crc;
+      t.frequency = ds_term->frequency;
+      memcpy ((char *) CTX->signature->data +
+              (i * sizeof (struct _ds_signature_token)), &t,
+              sizeof (struct _ds_signature_token));
+    }
+
+    /* If classification was provided, force probabilities */
+    if (CTX->classification == DSR_ISSPAM)
+      ds_term->s.probability = 1.00;
+    else if (CTX->classification == DSR_ISINNOCENT) 
+      ds_term->s.probability = 0.00;
+
+    if (ds_term->type == 'D' &&
+        ( CTX->training_mode != DST_TUM  || 
+          CTX->source == DSS_ERROR       ||
+          CTX->source == DSS_INOCULATION ||
+          ds_term->s.spam_hits + ds_term->s.innocent_hits < 50 ||
+          ds_term->key == diction->whitelist_token             ||
+          CTX->confidence < 0.70))
+    {
+        ds_term->s.status |= TST_DIRTY;
+    }
+
+    if (ds_term->type == 'B' &&
+        CTX->totals.innocent_learned + CTX->totals.innocent_classified > 500 &&
+        CTX->flags & DSF_NOISE &&
+        CTX->_sig_provided == 0)
+    {
+        ds_term->s.status |= TST_DIRTY;
+    }
+
+    /* SPAM */
+    if (CTX->result == DSR_ISSPAM)
+    {
+      /* Inoculations increase token count considerably */
+      if (CTX->source == DSS_INOCULATION)
+      {
+        if (ds_term->s.innocent_hits < 2 && ds_term->s.spam_hits < 5)
+          ds_term->s.spam_hits += 5;
+        else
+          ds_term->s.spam_hits += 2;
+      }
+
+      /* Standard increase */
+      else
+      {
+        if (CTX->flags & DSF_UNLEARN) {
+          if (CTX->classification == DSR_ISSPAM)
+          {
+            if (_ds_match_attribute(CTX->config->attributes, 
+              "ProcessorWordFrequency", "occurrence"))
+            {
+              ds_term->s.spam_hits -= ds_term->frequency;
+              if (ds_term->s.spam_hits < 0)
+                ds_term->s.spam_hits = 0;
+            } else {
+              ds_term->s.spam_hits -= (ds_term->s.spam_hits>0) ? 1:0;
+            }
+          }
+        } else {
+          if (_ds_match_attribute(CTX->config->attributes,
+             "ProcessorWordFrequency", "occurrence"))
+          {
+            ds_term->s.spam_hits += ds_term->frequency;
+          } else {
+            ds_term->s.spam_hits++;
+          }
+        }
+      }
+
+      if (SPAM_MISS(CTX) && 
+          !(CTX->flags & DSF_UNLEARN) && 
+          CTX->training_mode != DST_TOE &&
+          CTX->training_mode != DST_NOTRAIN) 
+      { 
+        if (_ds_match_attribute(CTX->config->attributes,
+           "ProcessorWordFrequency", "occurrence"))
+        {
+          ds_term->s.innocent_hits -= ds_term->frequency;
+          if (ds_term->s.innocent_hits < 0)
+            ds_term->s.innocent_hits = 0;
+        } else {
+          ds_term->s.innocent_hits -= (ds_term->s.innocent_hits>0) ? 1:0;
+        }
+      }
+    }
+
+    /* INNOCENT */
+    else
+    {
+      if (CTX->flags & DSF_UNLEARN) { 
+        if (CTX->classification == DSR_ISINNOCENT)
+        {
+          if (_ds_match_attribute(CTX->config->attributes,
+             "ProcessorWordFrequency", "occurrence"))
+          {
+            ds_term->s.innocent_hits -= ds_term->frequency;
+            if (ds_term->s.innocent_hits < 0)
+              ds_term->s.innocent_hits = 0;
+          } else {
+            ds_term->s.innocent_hits -= (ds_term->s.innocent_hits>0) ? 1:0;
+          }
+        }
+      } else {
+        if (_ds_match_attribute(CTX->config->attributes,
+         "ProcessorWordFrequency", "occurrence"))
+        {
+          ds_term->s.innocent_hits += ds_term->frequency;
+        } else {
+          ds_term->s.innocent_hits++;
+        }
+      }
+
+      if (FALSE_POSITIVE(CTX)         && 
+          !(CTX->flags & DSF_UNLEARN) && 
+          CTX->training_mode != DST_TOE &&
+          CTX->training_mode != DST_NOTRAIN)
+      {
+
+        if (_ds_match_attribute(CTX->config->attributes,
+          "ProcessorWordFrequency", "occurrence"))
+        {
+          ds_term->s.spam_hits -= ds_term->frequency;
+          if (ds_term->s.spam_hits < 0)
+            ds_term->s.spam_hits = 0;
+        } else {
+          ds_term->s.spam_hits -= (ds_term->s.spam_hits>0) ? 1:0;
+        }
+
+      }
+    }
+
+    ds_term = ds_diction_next(ds_c);
+    i++;
+  }
+  ds_diction_close(ds_c);
+  return 0;
+}
