@@ -1,4 +1,4 @@
-/* $Id: mysql_drv.c,v 1.59 2006/01/18 16:48:53 jonz Exp $ */
+/* $Id: mysql_drv.c,v 1.64 2006/02/02 17:13:11 jonz Exp $ */
 
 /*
  DSPAM
@@ -62,10 +62,8 @@
 #include "error.h"
 #include "language.h"
 #include "util.h"
-#ifdef PREFERENCES_EXTENSION
 #include "pref.h"
 #include "config_shared.h"
-#endif
 
 int test(MYSQL *, char *);
 
@@ -1094,6 +1092,11 @@ _ds_get_signature (DSPAM_CTX * CTX, struct _ds_spam_signature *SIG,
     /* Change the context's username and reinitialize storage */
 
     p = _mysql_drv_getpwuid (CTX, uid);
+    if (p == NULL) {
+      LOG(LOG_CRIT, "_ds_get_signature(): _mysql_drv_getpwuid(%d) failed: aborting", uid);
+      return EFAILURE;
+    }
+
     username = strdup(p->pw_name);
     _ds_shutdown_storage(CTX);
     free(CTX->username);
@@ -1590,6 +1593,7 @@ _mysql_drv_getpwnam (DSPAM_CTX * CTX, const char *name)
   MYSQL_RES *result;
   MYSQL_ROW row;
   char *virtual_table, *virtual_uid, *virtual_username;
+  char *sql_username;
 
   if ((virtual_table
     = _ds_read_attribute(CTX->config->attributes, "MySQLVirtualTable"))==NULL)
@@ -1613,9 +1617,20 @@ _mysql_drv_getpwnam (DSPAM_CTX * CTX, const char *name)
     s->p_getpwnam.pw_name = NULL;
   }
 
+  sql_username = malloc ((2 * strlen(name)) + 1);
+  if (sql_username == NULL)
+  {
+    return NULL;
+  }
+
+  mysql_real_escape_string (s->dbh, sql_username, name,
+                            strlen(name));
+
   snprintf (query, sizeof (query),
             "select %s from %s where %s = '%s'", 
-            virtual_uid, virtual_table, virtual_username, name);
+            virtual_uid, virtual_table, virtual_username, sql_username);
+
+  free (sql_username);
 
   if (MYSQL_RUN_QUERY (s->dbh, query))
   {
@@ -1789,6 +1804,7 @@ _mysql_drv_setpwnam (DSPAM_CTX * CTX, const char *name)
   char query[256];
   char *virtual_table, *virtual_uid, *virtual_username;
   struct _mysql_drv_storage *s = (struct _mysql_drv_storage *) CTX->storage;
+  char *sql_username;
 
   if ((virtual_table
     = _ds_read_attribute(CTX->config->attributes, "MySQLVirtualTable"))==NULL)
@@ -1811,9 +1827,20 @@ _mysql_drv_setpwnam (DSPAM_CTX * CTX, const char *name)
   }
 #endif
 
+  sql_username = malloc (strlen(name) * 2 + 1);
+  if (sql_username == NULL)
+  {
+    return NULL;
+  }
+
+  mysql_real_escape_string (s->dbh, sql_username, name,
+                            strlen(name));
+
   snprintf (query, sizeof (query),
             "insert into %s (%s, %s) values(NULL, '%s')",
-            virtual_table, virtual_uid, virtual_username, name);
+            virtual_table, virtual_uid, virtual_username, sql_username);
+
+  free (sql_username);
 
   /* we need to fail, to prevent a potential loop - even if it was inserted
    * by another process */
@@ -2254,77 +2281,6 @@ FAIL:
 
 }
 
-int _ds_pref_save(
-  config_t config,
-  const char *username,  
-  const char *home, 
-  agent_pref_t PTX,
-  void *dbh) 
-{
-  struct _mysql_drv_storage *s;
-  struct passwd *p;
-  char query[128];
-  DSPAM_CTX *CTX;
-  agent_attrib_t pref;
-  int uid, i = 0;
-  char m1[257], m2[257];
-
-  CTX = _mysql_drv_init_tools(home, config, dbh, DSM_PROCESS);
-  if (CTX == NULL) {
-    LOG (LOG_WARNING, "unable to initialize tools context");
-    return EUNKNOWN;
-  }
-
-  s = (struct _mysql_drv_storage *) CTX->storage;
-
-  if (username != NULL) {
-    p = _mysql_drv_getpwnam (CTX, username);
-
-    if (p == NULL)
-    {
-      LOGDEBUG ("_ds_pref_save: unable to _mysql_drv_getpwnam(%s)",
-              CTX->username);
-      dspam_destroy(CTX);
-      return EFAILURE;
-    } else {
-      uid = p->pw_uid;
-    }
-  } else {
-    uid = 0; /* Default Preferences */
-  }
-
-  snprintf(query, sizeof(query), "delete from dspam_preferences where uid = %d",
-     uid);
-
-  if (MYSQL_RUN_QUERY (s->dbh, query))
-  {
-    _mysql_drv_query_error (mysql_error (s->dbh), query);
-    dspam_destroy(CTX);
-    return EFAILURE;
-  }
-
-  for(i=0;PTX[i];i++) {
-    pref = PTX[i];
-
-    mysql_real_escape_string (s->dbh, m1, pref->attribute, 
-                              strlen(pref->attribute));
-    mysql_real_escape_string (s->dbh, m2, pref->value, strlen(pref->value));
-
-    snprintf(query, sizeof(query), "insert into dspam_preferences "
-      "(uid, attribute, value) values('%d', '%s', '%s')", uid, m1, m2); 
-
-    if (MYSQL_RUN_QUERY (s->dbh, query))
-    {
-      _mysql_drv_query_error (mysql_error (s->dbh), query);
-      dspam_destroy(CTX);
-      return EFAILURE;
-    }
-  }
-
-  dspam_destroy(CTX);
-  return 0;
-}
-
 int _mysql_drv_set_attributes(DSPAM_CTX *CTX, config_t config) {
   int i, ret = 0;
   attribute_t t;
@@ -2362,328 +2318,28 @@ int _mysql_drv_set_attributes(DSPAM_CTX *CTX, config_t config) {
   return 0;
 }
 
+#else
+/* Preference Stubs for Flat-File */
+
+agent_pref_t _ds_pref_load(config_t config, const char *user,
+  const char *home, void *dbh)
+{
+  return _ds_ff_pref_load(config, user, home, dbh);
+}
+
+int _ds_pref_set(config_t config, const char *user, const char *home,
+  const char *attrib, const char *value, void *dbh)
+{
+  return _ds_ff_pref_set(config, user, home, attrib, value, dbh);
+}
+
+int _ds_pref_del(config_t config, const char *user, const char *home,
+  const char *attrib, void *dbh)
+{
+  return _ds_pref_del(config, user, home, attrib, dbh);
+}
 #endif
 
-/* Neural network functions */
-
-int _ds_get_node(DSPAM_CTX * CTX, char *user, struct _ds_neural_record *node) {
-  char query[256];
-  struct _mysql_drv_storage *s = (struct _mysql_drv_storage *) CTX->storage;
-  struct passwd *p;
-  MYSQL_RES *result;
-  MYSQL_ROW row;
-  uid_t n_uid;
-
-  if (user == NULL) 
-    n_uid = node->uid;
-  else {
-    p = _mysql_drv_getpwnam (CTX, user);
-    if (p == NULL)
-    {
-      LOGDEBUG ("_mysql_drv_get_spamtotals: unable to _mysql_drv_getpwnam(%s)",
-                user);
-      return EINVAL;
-    }
-    n_uid = p->pw_uid;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _mysql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _mysql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL) 
-  {
-    LOGDEBUG ("_mysql_drv_get_spamtotals: unable to _mysql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  snprintf(query, sizeof(query), "select total_correct, total_incorrect from "
-    "dspam_neural_data where uid = %d and node = %d", (int) p->pw_uid,
-    (int) n_uid);
-
-  if (MYSQL_RUN_QUERY (s->dbh, query))
-  {
-    _mysql_drv_query_error (mysql_error (s->dbh), query);
-    return EUNKNOWN;
-  }
-
-  result = mysql_use_result (s->dbh);
-  if (result == NULL)
-    goto NEWNODE;
-
-  row = mysql_fetch_row (result);
-  if (row == NULL)
-  {
-    mysql_free_result (result);
-    goto NEWNODE;
-  }
-  
-  node->uid = n_uid;
-  node->total_correct = strtol(row[0], NULL, 0);
-  node->total_incorrect = strtol(row[1], NULL, 0);
-  node->control_correct = node->total_correct;
-  node->control_incorrect = node->total_incorrect;
-  node->disk = 'Y';
-
-  mysql_free_result (result);
-
-  return 0;
-
-NEWNODE:
-  node->uid = n_uid;
-  node->total_correct = 0;
-  node->total_incorrect = 0;
-  node->control_correct = 0;
-  node->control_incorrect = 0;
-  node->disk = 'N';
-  return 0;
-}
-
-int _ds_set_node (DSPAM_CTX * CTX, char *user, struct _ds_neural_record *node) {
-  char query[256];
-  struct _mysql_drv_storage *s = (struct _mysql_drv_storage *) CTX->storage;
-  struct passwd *p;
-  uid_t n_uid;
-                                                                                                                              
-  if (user == NULL)
-    n_uid = node->uid;
-  else {
-    p = _mysql_drv_getpwnam (CTX, user);
-    if (p == NULL)
-    {
-      LOGDEBUG ("_mysql_drv_get_spamtotals: unable to _mysql_drv_getpwnam(%s)",
-                user);
-      return EINVAL;
-    }
-    n_uid = p->pw_uid;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _mysql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _mysql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL)
-  {
-    LOGDEBUG ("_mysql_drv_get_spamtotals: unable to _mysql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  if (node->disk == 'N') {
-    snprintf(query, sizeof(query), "insert into dspam_neural_data "
-     "(uid, node, total_correct, total_incorrect) values ("
-     "%d, %d, %ld, %ld)", (int) p->pw_uid, (int) node->uid, node->total_correct,
-        node->total_incorrect);
-
-    if (MYSQL_RUN_QUERY (s->dbh, query))
-      node->disk = 'Y';
-  }
-
-  if (node->disk == 'Y') {
-    snprintf(query, sizeof(query), "update dspam_neural_data set "
-      "total_correct = total_correct + %ld, "
-      "total_incorrect = total_incorrect + %ld "
-      "where uid = %d and node = %d", 
-        node->total_correct - node->control_correct,
-        node->total_incorrect - node->control_incorrect,
-      (int) p->pw_uid, (int) node->uid);
-
-    if (MYSQL_RUN_QUERY (s->dbh, query))
-    {
-      _mysql_drv_query_error (mysql_error (s->dbh), query);
-      return EUNKNOWN;
-    }
-  }
-
-  node->disk = 'Y';
-                                                                                
-  return 0;
-
-}
-
-int
-_ds_get_decision (DSPAM_CTX * CTX, struct _ds_neural_decision *DEC,
-                  const char *signature)
-{
-  struct _mysql_drv_storage *s = (struct _mysql_drv_storage *) CTX->storage;
-  struct passwd *p;
-  unsigned long *lengths;
-  char *mem;
-  char query[128];
-  MYSQL_RES *result;
-  MYSQL_ROW row;
-
-  if (s->dbh == NULL)
-  {
-    LOGDEBUG ("_ds_get_decision: invalid database handle (NULL)");
-    return EINVAL;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _mysql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _mysql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL)
-  {
-    LOGDEBUG ("_ds_get_decision: unable to _mysql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  snprintf (query, sizeof (query),
-    "select data, length from dspam_neural_decisions where uid = %d "
-    "and signature = \"%s\"",
-            (int) p->pw_uid, signature);
-  if (mysql_real_query (s->dbh, query, strlen (query)))
-  {
-    _mysql_drv_query_error (mysql_error (s->dbh), query);
-    return EFAILURE;
-  }
-
-  result = mysql_use_result (s->dbh);
-  if (result == NULL) {
-    LOGDEBUG("mysql_use_result() failed in _ds_get_decision"); 
-    return EFAILURE;
-  }
-
-  row = mysql_fetch_row (result);
-  if (row == NULL)
-  {
-    mysql_free_result (result);
-    LOGDEBUG("mysql_fetch_row() failed in _ds_get_decision");
-    return EFAILURE;
-  }
-
-  lengths = mysql_fetch_lengths (result);
-  if (lengths == NULL || lengths[0] == 0)
-  {
-    mysql_free_result (result);
-    LOGDEBUG("mysql_fetch_lengths() failed in _ds_get_decision");
-    return EFAILURE;
-  }
-
-  mem = malloc (lengths[0]);
-  if (mem == NULL)
-  {
-    LOG (LOG_CRIT, ERR_MEM_ALLOC);
-    mysql_free_result (result);
-    return EUNKNOWN;
-  }
-
-  memcpy (mem, row[0], lengths[0]);
-  DEC->data = mem;
-  DEC->length = strtol (row[1], NULL, 0);
-
-  mysql_free_result (result);
-
-  return 0;
-}
-
-int
-_ds_set_decision (DSPAM_CTX * CTX, struct _ds_neural_decision *DEC,
-                   const char *signature)
-{
-  struct _mysql_drv_storage *s = (struct _mysql_drv_storage *) CTX->storage;
-  unsigned long length;
-  char *mem;
-  char scratch[1024];
-  buffer *query;
-  struct passwd *p;
-
-  if (s->dbh == NULL)
-  {
-    LOGDEBUG ("_ds_set_decision; invalid database handle (NULL)");
-    return EINVAL;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _mysql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _mysql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL)
-  {
-    LOGDEBUG ("_ds_set_decision: unable to _mysql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  query = buffer_create (NULL);
-  if (query == NULL)
-  {
-    LOG (LOG_CRIT, ERR_MEM_ALLOC);
-    return EUNKNOWN;
-  }
-
-  mem = malloc (DEC->length * 2);
-  if (mem == NULL)
-  {
-    LOG (LOG_CRIT, ERR_MEM_ALLOC);
-    buffer_destroy(query);
-    return EUNKNOWN;
-  }
-  length = mysql_real_escape_string (s->dbh, mem, DEC->data, DEC->length);
-
-  snprintf (scratch, sizeof (scratch),
-            "insert into dspam_neural_decisions(uid, signature, length, created_on, data) values(%d, \"%s\", %ld, current_date(), \"",
-            (int) p->pw_uid, signature, DEC->length);
-  buffer_cat (query, scratch);
-  buffer_cat (query, mem);
-  buffer_cat (query, "\")");
-
-  if (mysql_real_query (s->dbh, query->data, query->used))
-  {
-    _mysql_drv_query_error (mysql_error (s->dbh), query->data);
-    buffer_destroy(query);
-    free(mem);
-    return EFAILURE;
-  }
-
-  free (mem);
-  return 0;
-}
-
-int
-_ds_delete_decision (DSPAM_CTX * CTX, const char *signature)
-{
-  struct _mysql_drv_storage *s = (struct _mysql_drv_storage *) CTX->storage;
-  struct passwd *p;
-  char query[128];
-                                                                                                                              
-  if (s->dbh == NULL)
-  {
-    LOGDEBUG ("_ds_delete_decision: invalid database handle (NULL)");
-    return EINVAL;
-  }
-                                                                                                                              
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _mysql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _mysql_drv_getpwnam (CTX, CTX->group);
-                                                                                                                              
-  if (p == NULL)
-  {
-    LOGDEBUG ("_ds_delete_decision: unable to _mysql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-                                                                                                                              
-  snprintf (query, sizeof (query),
-    "delete from dspam_neural_decisions where uid = %d and signature = \"%s\"",
-            (int) p->pw_uid, signature);
-  if (MYSQL_RUN_QUERY (s->dbh, query))
-  {
-    _mysql_drv_query_error (mysql_error (s->dbh), query);
-    return EFAILURE;
-  }
-                                                                                                                              
-  return 0;
-}
-
 void *_ds_connect (DSPAM_CTX *CTX)
 {
   return (void *) _mysql_drv_connect(CTX);
@@ -2697,17 +2353,20 @@ MYSQL *_mysql_drv_connect (DSPAM_CTX *CTX)
   char buffer[128];
   char hostname[128] = { 0 };
   char user[64] = { 0 };
-  char password[32] = { 0 };
+  char password[64] = { 0 };
   char db[64] = { 0 };
   int port = 3306, i = 0, real_connect_flag = 0;
+  char *p;
 
   /* Read storage attributes */
-  if (_ds_read_attribute(CTX->config->attributes, "MySQLServer")) {
-    char *p;
+  if ((p = _ds_read_attribute(CTX->config->attributes, "MySQLServer"))) {
 
-    strlcpy(hostname, 
-            _ds_read_attribute(CTX->config->attributes, "MySQLServer"), 
-            sizeof(hostname));
+    strlcpy(hostname, p, sizeof(hostname));
+    if (strlen(p) >= sizeof(hostname))
+    {
+      LOG(LOG_WARNING, "Truncating MySQLServer to %d characters.",
+          sizeof(hostname)-1);
+    }
 
     if (_ds_read_attribute(CTX->config->attributes, "MySQLPort"))
       port = atoi(_ds_read_attribute(CTX->config->attributes, "MySQLPort"));
@@ -2715,11 +2374,32 @@ MYSQL *_mysql_drv_connect (DSPAM_CTX *CTX)
       port = 0;
 
     if ((p = _ds_read_attribute(CTX->config->attributes, "MySQLUser")))
+    {
       strlcpy(user, p, sizeof(user));
+      if (strlen(p) >= sizeof(user))
+      {
+        LOG(LOG_WARNING, "Truncating MySQLUser to %d characters.",
+            sizeof(user)-1);
+      }
+    }
     if ((p = _ds_read_attribute(CTX->config->attributes, "MySQLPass")))
+    {
       strlcpy(password, p, sizeof(password));
+      if (strlen(p) >= sizeof(password))
+      {
+        LOG(LOG_WARNING, "Truncating MySQLPass to %d characters.",
+            sizeof(password)-1);
+      }
+    }
     if ((p = _ds_read_attribute(CTX->config->attributes, "MySQLDb")))
+    {
       strlcpy(db, p, sizeof(db)); 
+      if (strlen(p) >= sizeof(db))
+      {
+        LOG(LOG_WARNING, "Truncating MySQLDb to %d characters.",
+            sizeof(db)-1);
+      }
+    }
 
     if (_ds_match_attribute(CTX->config->attributes, "MySQLCompress", "true"))
       real_connect_flag = CLIENT_COMPRESS;
