@@ -1,4 +1,4 @@
-/* $Id: pgsql_drv.c,v 1.49 2006/01/18 16:48:53 jonz Exp $ */
+/* $Id: pgsql_drv.c,v 1.53 2006/02/02 17:13:11 jonz Exp $ */
 
 /*
  DSPAM
@@ -62,9 +62,7 @@
 #include "error.h"
 #include "language.h"
 #include "util.h"
-#ifdef PREFERENCES_EXTENSION
 #include "pref.h"
-#endif
 
 
 #ifdef HAVE_PQFREEMEM
@@ -1134,6 +1132,10 @@ _ds_get_signature (DSPAM_CTX * CTX, struct _ds_spam_signature *SIG,
     /* Change the context's username and reinitialize storage */
 
     p = _pgsql_drv_getpwuid (CTX, uid);
+    if (!p) {
+      LOG(LOG_CRIT, "_ds_get_signature(): _mysql_drv_getpwuid(%d) failed: aborting", uid);
+      return EFAILURE;
+    }
     username = strdup(p->pw_name);
     _ds_shutdown_storage(CTX);
     free(CTX->username);
@@ -2376,87 +2378,6 @@ FAIL:
   return EFAILURE;
 }
 
-int _ds_pref_save(
-  config_t config,
-  const char *username, 
-  const char *home, 
-  agent_pref_t PTX, 
-  void *dbh) 
-{
-  struct _pgsql_drv_storage *s;
-  struct passwd *p;
-  char query[128];
-  DSPAM_CTX *CTX;
-  agent_attrib_t pref;
-  int uid, i = 0;
-  unsigned char *m1, *m2;
-  size_t length;
-  PGresult *result;
-
-  CTX = _pgsql_drv_init_tools(home, config, dbh, DSM_PROCESS);
-  if (CTX == NULL)
-  {
-    LOG (LOG_WARNING, "unable to initialize tools context");
-    return EFAILURE;
-  }
-
-  s = (struct _pgsql_drv_storage *) CTX->storage;
-
-  if (username != NULL) {
-    p = _pgsql_drv_getpwnam (CTX, username);
-
-    if (p == NULL)
-    {
-      LOGDEBUG ("_ds_pref_save: unable to _pgsql_drv_getpwnam(%s)",
-              CTX->username);
-      dspam_destroy(CTX);
-      return EFAILURE;
-    } else {
-      uid = p->pw_uid;
-    }
-  } else {
-    uid = 0; /* Default Preferences */
-  }
-
-  snprintf(query, sizeof(query), "DELETE FROM dspam_preferences "
-                                 "WHERE uid = '%d'", uid);
-
-  result = PQexec(s->dbh, query);
-  if ( !result || PQresultStatus(result) != PGRES_COMMAND_OK )
-  {
-    _pgsql_drv_query_error (PQresultErrorMessage(result), query);
-    if (result) PQclear(result);
-    dspam_destroy(CTX);
-    return EFAILURE;
-  }
-  PQclear(result);
-  
-  for(i=0;PTX[i];i++) {
-    pref = PTX[i];
-
-    m1 = PQescapeBytea((unsigned char *) pref->attribute, strlen(pref->attribute), &length);
-    m2 = PQescapeBytea((unsigned char *) pref->value, strlen(pref->value), &length);
-
-    snprintf(query, sizeof(query), "INSERT INTO dspam_preferences "
-      "(uid, attribute, value) VALUES ('%d', '%s', '%s')", uid, m1, m2);
-
-    PQFREEMEM(m1);
-    PQFREEMEM(m2);
-    result = PQexec(s->dbh, query);
-    if ( !result || PQresultStatus(result) != PGRES_COMMAND_OK )
-    {
-      _pgsql_drv_query_error (PQresultErrorMessage(result), query);
-      if (result) PQclear(result);
-      dspam_destroy(CTX);
-      return EFAILURE;
-    }
-    PQclear(result);
-  }
-  
-  dspam_destroy(CTX);
-  return 0;
-}
-
 int _pgsql_drv_set_attributes(DSPAM_CTX *CTX, config_t config) {
   int i, ret = 0;
   attribute_t t;
@@ -2494,327 +2415,29 @@ int _pgsql_drv_set_attributes(DSPAM_CTX *CTX, config_t config) {
   return 0;
 }
 
+#else
+/* Preference Stubs for Flat-File */
+
+agent_pref_t _ds_pref_load(config_t config, const char *user,
+  const char *home, void *dbh)
+{
+  return _ds_ff_pref_load(config, user, home, dbh);
+}
+
+int _ds_pref_set(config_t config, const char *user, const char *home,
+  const char *attrib, const char *value, void *dbh)
+{
+  return _ds_ff_pref_set(config, user, home, attrib, value, dbh);
+}
+
+int _ds_pref_del(config_t config, const char *user, const char *home,
+  const char *attrib, void *dbh)
+{
+  return _ds_pref_del(config, user, home, attrib, dbh);
+}
+
 #endif
 
-/* Neural network functions */
-
-int _ds_get_node(DSPAM_CTX * CTX, char *user, struct _ds_neural_record *node) {
-  char query[256];
-  struct _pgsql_drv_storage *s = (struct _pgsql_drv_storage *) CTX->storage;
-  struct passwd *p;
-  PGresult *result;
-  uid_t n_uid;
-
-  if (user == NULL)
-    n_uid = node->uid;
-  else {
-    p = _pgsql_drv_getpwnam (CTX, user);
-    if (p == NULL)
-    {
-      LOGDEBUG ("_ds_get_node: unable to _pgsql_drv_getpwnam(%s)",
-                user);
-      return EINVAL;
-    }
-    n_uid = p->pw_uid;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _pgsql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _pgsql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL)
-  {
-    LOGDEBUG ("_ds_get_node: unable to _pgsql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  snprintf(query, sizeof(query), "SELECT total_correct, total_incorrect FROM "
-           "dspam_neural_data WHERE uid = '%d' AND node = '%d'", p->pw_uid, n_uid);
-
-  result = PQexec(s->dbh, query);
-  if ( !result || PQresultStatus(result) != PGRES_TUPLES_OK )
-  {
-    _pgsql_drv_query_error (PQresultErrorMessage(result), query);
-    if (result) PQclear(result);
-    return EUNKNOWN;
-  }
-
-  if ( PQntuples(result) < 1 )
-  {
-    if (result) PQclear(result);
-    goto NEWNODE;
-  }
-
-  node->uid = n_uid;
-  node->total_correct = strtol (PQgetvalue( result,0,0), NULL, 0);
-  node->total_incorrect = strtol(PQgetvalue( result,0,1), NULL, 0);
-  node->control_correct = node->total_correct;
-  node->control_incorrect = node->total_incorrect;
-  node->disk = 'Y';
-
-  if (result) PQclear(result);
-
-  return 0;
-
-NEWNODE:
-  node->uid = n_uid;
-  node->total_correct = 0;
-  node->total_incorrect = 0;
-  node->control_correct = 0;
-  node->control_incorrect = 0;
-  node->disk = 'N';
-  return 0;
-}
-
-int _ds_set_node (DSPAM_CTX * CTX, char *user, struct _ds_neural_record *node) {
-  char query[256];
-  struct _pgsql_drv_storage *s = (struct _pgsql_drv_storage *) CTX->storage;
-  struct passwd *p;
-  uid_t n_uid;
-  PGresult *result;
-
-  if (user == NULL)
-    n_uid = node->uid;
-  else {
-    p = _pgsql_drv_getpwnam (CTX, user);
-    if (p == NULL)
-    {
-      LOGDEBUG ("_ds_set_node: unable to _pgsql_drv_getpwnam(%s)",
-                user);
-      return EINVAL;
-    }
-    n_uid = p->pw_uid;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _pgsql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _pgsql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL)
-  {
-    LOGDEBUG ("_ds_set_node: unable to _pgsql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  if (node->disk == 'N') {
-    snprintf(query, sizeof(query), "INSERT INTO dspam_neural_data "
-             "(uid, node, total_correct, total_incorrect) VALUES ("
-             "%d, %d, %ld, %ld)", p->pw_uid, node->uid, node->total_correct,
-             node->total_incorrect);
-
-    result = PQexec(s->dbh, query);
-    if ( PQresultStatus(result) != PGRES_COMMAND_OK )
-      node->disk = 'Y';
-    PQclear(result);
-  }
-
-  if (node->disk == 'Y') {
-    snprintf(query, sizeof(query), "UPDATE dspam_neural_data SET "
-             "total_correct = total_correct + %ld, "
-             "total_incorrect = total_incorrect + %ld "
-             "WHERE uid = '%d' AND node = '%d'",
-             node->total_correct - node->control_correct,
-             node->total_incorrect - node->control_incorrect,
-             p->pw_uid, node->uid);
-
-    result = PQexec(s->dbh, query);
-    if ( !result || PQresultStatus(result) != PGRES_COMMAND_OK )
-    {
-      _pgsql_drv_query_error (PQresultErrorMessage(result), query);
-      if (result) PQclear(result);
-      return EUNKNOWN;
-    }
-    PQclear(result);
-  }
-
-  node->disk = 'Y';
-
-  return 0;
-
-}
-
-int
-_ds_get_decision (DSPAM_CTX * CTX, struct _ds_neural_decision *DEC,
-                  const char *signature)
-{
-  struct _pgsql_drv_storage *s = (struct _pgsql_drv_storage *) CTX->storage;
-  struct passwd *p;
-  unsigned char *mem, *mem2;
-  char query[128];
-  size_t length;
-  PGresult *result;
-
-  if (s->dbh == NULL)
-  {
-    LOGDEBUG ("_ds_get_decision: invalid database handle (NULL)");
-    return EINVAL;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _pgsql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _pgsql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL)
-  {
-    LOGDEBUG ("_ds_get_decision: unable to _pgsql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  snprintf (query, sizeof (query),
-            "SELECT data, length FROM dspam_neural_decisions WHERE uid = '%d' AND signature = '%s'",
-            p->pw_uid, signature);
-
-  result = PQexec(s->dbh, query);
-  if ( !result || PQresultStatus(result) != PGRES_TUPLES_OK )
-  {
-    _pgsql_drv_query_error (PQresultErrorMessage(result), query);
-    if (result) PQclear(result);
-    return EFAILURE;
-  }
-
-  if ( PQntuples(result) < 1 )
-  {
-    if (result) PQclear(result);
-    return EFAILURE;
-  }
-
-  if ( PQgetlength(result, 0, 0) == 0)
-  {
-    if (result) PQclear(result);
-    return EFAILURE;
-  }
-
-  mem = PQunescapeBytea((unsigned char *) PQgetvalue(result,0,0), &length);
-  DEC->length = strtol (PQgetvalue(result,0,1), NULL, 0);
-
-  mem2 = calloc(1, length+1);
-  if (!mem2) {
-    PQFREEMEM(mem);
-    LOG(LOG_CRIT, ERR_MEM_ALLOC);
-    return EUNKNOWN;
-  }
-
-  memcpy(mem2, mem, length);
-  DEC->data = mem2;
-  PQFREEMEM(mem);
-
-  if (result) PQclear(result);
-
-  return 0;
-}
-
-int
-_ds_set_decision (DSPAM_CTX * CTX, struct _ds_neural_decision *DEC,
-                   const char *signature)
-{
-  struct _pgsql_drv_storage *s = (struct _pgsql_drv_storage *) CTX->storage;
-  unsigned char *mem;
-  char scratch[1024];
-  buffer *query;
-  struct passwd *p;
-  size_t length;
-  PGresult *result;
-
-  if (s->dbh == NULL)
-  {
-    LOGDEBUG ("_ds_set_decision; invalid database handle (NULL)");
-    return EINVAL;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _pgsql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _pgsql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL)
-  {
-    LOGDEBUG ("_ds_set_decision: unable to _pgsql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  query = buffer_create (NULL);
-  if (query == NULL)
-  {
-    LOG (LOG_CRIT, ERR_MEM_ALLOC);
-    return EUNKNOWN;
-  }
-
-  mem = PQescapeBytea(DEC->data, DEC->length, &length);
-
-  snprintf (scratch, sizeof (scratch),
-            "INSERT INTO dspam_neural_decisions (uid, signature, length, created_on, data) VALUES (%d, '%s', %ld, CURRENT_DATE, '",
-            p->pw_uid, signature, DEC->length);
-  buffer_cat (query, scratch);
-  buffer_cat (query, (const char *) mem);
-  buffer_cat (query, "')");
-
-  result = PQexec(s->dbh, query->data);
-  if ( !result || PQresultStatus(result) != PGRES_COMMAND_OK )
-  {
-    _pgsql_drv_query_error (PQresultErrorMessage(result), query->data);
-    if (result) PQclear(result);
-    buffer_destroy(query);
-    PQFREEMEM(mem);
-    return EFAILURE;
-  }
-
-  if (result) PQclear(result);
-  buffer_destroy(query);
-  PQFREEMEM(mem);
-  return 0;
-}
-
-int
-_ds_delete_decision (DSPAM_CTX * CTX, const char *signature)
-{
-  struct _pgsql_drv_storage *s = (struct _pgsql_drv_storage *) CTX->storage;
-  struct passwd *p;
-  char query[128];
-  PGresult *result;
-
-  if (s->dbh == NULL)
-  {
-    LOGDEBUG ("_ds_delete_decision: invalid database handle (NULL)");
-    return EINVAL;
-  }
-
-  if (!CTX->group || CTX->flags & DSF_MERGED)
-    p = _pgsql_drv_getpwnam (CTX, CTX->username);
-  else
-    p = _pgsql_drv_getpwnam (CTX, CTX->group);
-
-  if (p == NULL)
-  {
-    LOGDEBUG ("_ds_delete_decision: unable to _pgsql_drv_getpwnam(%s)",
-              CTX->username);
-    return EINVAL;
-  }
-
-  snprintf (query, sizeof (query),
-            "DELETE FROM dspam_neural_decisions WHERE uid = '%d' AND signature = '%s'",
-            p->pw_uid, signature);
-
-  result = PQexec(s->dbh, query);
-  if ( !result || PQresultStatus(result) != PGRES_COMMAND_OK )
-  {
-    _pgsql_drv_query_error (PQresultErrorMessage(result), query);
-    if (result) PQclear(result);
-    return EFAILURE;
-  }
-  PQclear(result);
-
-  return 0;
-}
-
-/* END Neural network functions */
-
 void *_ds_connect (DSPAM_CTX *CTX)
 {
   return (void *) _pgsql_drv_connect(CTX);
